<?php
/***************************************************************************
 *                                                                          *
 *				   (c) 2013 Re:Desk Development Team    				   *
 *                                                                          *
 * This  is  commercial  software,  only  users  who have purchased a valid *
 * license  and  accept  to the terms of the  License Agreement can install *
 * and use this program.                                                    *
 *                                                                          *
 ****************************************************************************
 * PLEASE READ THE FULL TEXT  OF THE SOFTWARE  LICENSE   AGREEMENT  IN  THE *
 * "copyright.txt" FILE PROVIDED WITH THIS DISTRIBUTION PACKAGE.            *
 ****************************************************************************/

class HDController extends RController
{
	// special buttons
	public $show_refresh = false;
	public $show_selected = false;
	public $show_select = false;
	public $show_direct = false;

	public $total_found = false;
	public $total_my = false;
	public $total_trash = false;
	public $total_list_found = false;
	public $total_open = false;
	public $total_twitter = false;
	public $total_facebook = false;

	public $load_more = false;
	public $per_page;

	public $ajax_updates = array ();
	public $ajax_data = array ();

	public $header_content = false;
	public $left_content = false;

	public $ticket_selected = false;
	public $is_redirect = false;

	private $_pageTitle = '';

	/**
	 * @var AppDbAuthManager $manager
	 */
	public $manager;

	/**
	 * @var User $user
	 */
	public $user = false;

	public $params;

	public function init()
	{
		$this->params['per_page'] = Yii::app()->request->getParam('per_page');

		if ($this->block_helpdesk && $this->getUniqueId() == 'ticket')
			Yii::app()->request->redirect('/site/unpaid', true);

		/**
		 * @var User $user
		 */
		$user = Yii::app()->getModule('user')->user();
		if ($user)
			$this->user = $user;

		if (empty($_REQUEST['folder']))
			$_REQUEST['folder'] = TicketFolders::INBOX;

		if (empty($_REQUEST['filter']))
			$_REQUEST['filter'] = '';

		if (!empty($_REQUEST['cc']))
			Yii::app()->cache->flush();

		if (!empty($_REQUEST['refresh']) && $this->user && $this->user->is_worker)
			$this->_fetchMails();

		$this->total_found = Ticket::model()->count(Ticket::getBaseCriteria('', 'unread', '', TicketFolders::INBOX));
		$this->total_my = Ticket::model()->count(Ticket::getBaseCriteria('my', 'unread', '', $_REQUEST['folder']));
		$this->total_trash = Ticket::model()->count(Ticket::getBaseCriteria('my', 'unread', '', TicketFolders::TRASH));
		$this->total_twitter = Ticket::model()->count(Ticket::getBaseCriteria('my', 'unread', '', TicketFolders::TWITTER));
		$this->total_facebook = Ticket::model()->count(Ticket::getBaseCriteria('my', 'unread', '', TicketFolders::FACEBOOK));

		$this->total_open = Ticket::model()->count(Ticket::getBaseCriteria($_REQUEST['filter'], 'open', '', $_REQUEST['folder']));

		$this->manager = Yii::app()->getAuthManager();

		if (Settings::get('use_ssl') && (!Yii::app()->request->isSecureConnection))
		{
			$url = 'https://' . Yii::app()->request->serverName . Yii::app()->request->url;
			Yii::app()->request->redirect($url, true);
		}

		if (is_readable($this->getViewPath() . '/_header.php')) {
			$this->header_content = $this->renderPartial('_header', null, true);
		}

		if ($this->user && is_readable($this->getViewPath() . '/_left.php')) {
			$this->left_content = $this->renderPartial('_left_wrapper', null, true);
		}

		$this->isStaffWorking();
	}

	public function checkUpgrade()
	{
		if ($this->check_upgrade)
		{
			$append = http_build_query(array ('ln' => Settings::get('license_number'), 'u' => $this->total_users));
			return "<img style='display: none;' src='https://re-desk.com/index.php?dispatch=redesk.check_upgrade&" . $append . "' />";
		}
		else
			return '';
	}

	public function redirect($url,$terminate=true,$statusCode=302)
	{
		$this->is_redirect = true;
		parent::redirect($url, $terminate, $statusCode);
	}

	public function getPageTitle()
	{
		$title = parent::getPageTitle();
		$titles = explode(' - ', $title);

		if (count($titles) > 1)
		{
			foreach ($titles as $k => &$sub_title)
			{
				if ($k > 0)
					$sub_title = _t(mb_strtolower($sub_title, APP_ENCODING));
			}
		}

		return implode(' - ', $titles);
	}

	public function disabledFeature()
	{
		if (Yii::app()->cache && Yii::app()->cache->get('expired_license'))
			return true;
		else
			return false;
	}

	public function isStaffWorking()
	{
		if (Settings::get('worktime'))
		{
			list($from, $to) = explode('|', str_replace(':', '.', Settings::get('worktime')));
			$current = date('H.i');

			if ($current >= $from && $current <= $to || (intval($from) == 0 && intval($to) == 0))
				return true;

			return false;
		}
		else
			return true;
	}

	public function createAbsoluteUrl($route,$params=array(),$schema='',$ampersand='&')
    {
        $url = parent::createAbsoluteUrl($route,$params,$schema,$ampersand);

        return str_replace(':443', '', $url);
    }

	private function _fetchMails()
	{
		define('CONSOLE_APP', true);
		$commandPath = Yii::app()->getBasePath() . DIRECTORY_SEPARATOR . 'commands';
		$runner = new CConsoleCommandRunner();
		$runner->addCommands($commandPath);
		$commandPath = Yii::getFrameworkPath() . DIRECTORY_SEPARATOR . 'cli' . DIRECTORY_SEPARATOR . 'commands';
		$runner->addCommands($commandPath);
		$args = array('yiic', 'mailretriver');
		$runner->run($args);

		return true;
	}
}