<?php
/***************************************************************************
 *                                                                          *
 *				   (c) 2013 Re:Desk Development Team    				   *
 *                                                                          *
 * This  is  commercial  software,  only  users  who have purchased a valid *
 * license  and  accept  to the terms of the  License Agreement can install *
 * and use this program.                                                    *
 *                                                                          *
 ****************************************************************************
 * PLEASE READ THE FULL TEXT  OF THE SOFTWARE  LICENSE   AGREEMENT  IN  THE *
 * "copyright.txt" FILE PROVIDED WITH THIS DISTRIBUTION PACKAGE.            *
 ****************************************************************************/

class SettingsController extends HDController
{
	/**
	 * @var string the default layout for the views. Defaults to '//layouts/column2', meaning
	 * using two-column layout. See 'protected/views/layouts/column2.php'.
	 */
	public $layout='//layouts/column2';
	public $show_direct = true;

	/**
	 * @return array action filters
	 */
	public function filters()
	{
		return array(
			'rights', // perform access control for CRUD operations
		);
	}

	/**
	 * Specifies the access control rules.
	 * This method is used by the 'accessControl' filter.
	 * @return array access control rules
	 */
	public function accessRules()
	{
		return array(
			array('allow',  // allow all users to perform 'index' and 'view' actions
				'actions'=>array('index','view'),
				'users'=>array('*'),
			),
			array('allow', // allow authenticated user to perform 'create' and 'update' actions
				'actions'=>array('create','update'),
				'users'=>array('@'),
			),
			array('allow', // allow admin user to perform 'admin' and 'delete' actions
				'actions'=>array('admin','delete'),
				'users'=>array('admin'),
			),
			array('deny',  // deny all users
				'users'=>array('*'),
			),
		);
	}

	/**
	 * Displays a particular model.
	 * @param integer $id the ID of the model to be displayed
	 */
	public function actionView($id)
	{
		$this->render('view',array(
			'model'=>$this->loadModel($id),
		));
	}

	/**
	 * Creates a new model.
	 * If creation is successful, the browser will be redirected to the 'view' page.
	 */
	public function actionCreate()
	{
		$model=new Settings;

		// Uncomment the following line if AJAX validation is needed
		// $this->performAjaxValidation($model);

		if(isset($_POST['Settings']))
		{
			$model->attributes=$_POST['Settings'];
			if($model->save())
				$this->redirect(array('view','id'=>$model->id));
		}

		$this->render('create',array(
			'model'=>$model,
		));
	}

	/**
	 * Updates a particular model.
	 * If update is successful, the browser will be redirected to the 'view' page.
	 * @param integer $id the ID of the model to be updated
	 */
	public function actionUpdate($id)
	{
		$model=$this->loadModel($id);

		// Uncomment the following line if AJAX validation is needed
		// $this->performAjaxValidation($model);

		if(isset($_POST['Settings']))
		{
			$model->attributes=$_POST['Settings'];
			if($model->save())
				$this->redirect(array('view','id'=>$model->id));
		}

		$this->render('update',array(
			'model'=>$model,
		));
	}

	/**
	 * Deletes a particular model.
	 * If deletion is successful, the browser will be redirected to the 'admin' page.
	 * @param integer $id the ID of the model to be deleted
	 */
	public function actionDelete($id)
	{
		if(Yii::app()->request->isPostRequest)
		{
			// we only allow deletion via POST request
			$this->loadModel($id)->delete();

			// if AJAX request (triggered by deletion via admin grid view), we should not redirect the browser
			if(!isset($_GET['ajax']))
				$this->redirect(isset($_POST['returnUrl']) ? $_POST['returnUrl'] : array('admin'));
		}
		else
			throw new CHttpException(400, _t('invalid request'));
	}

	/**
	 * Lists all models.
	 */
	public function actionIndex()
	{
		if (!empty($_POST['settings']))
		{
			// prepare staff time
			$from = intval($_REQUEST['from']);
			$to = intval($_REQUEST['to']);

			if (!$from && !$to)
				$_POST['settings']['worktime'] = '';
			else
				$_POST['settings']['worktime'] = "$_REQUEST[from]|$_REQUEST[to]";

			foreach ($_POST['settings'] as $setting_label => $setting_value)
			{
				/**
				 * @var Settings $setting
				 */
				$setting = Settings::model()->findByAttributes(array('label' => $setting_label));

				if ($setting_label == 'languages')
				{
					if (empty($setting_value))
					{
						Yii::app()->user->setFlash('error', _t('languages error'));
					}
					else
					{
						$setting->avail_values = CJSON::encode($setting_value);

						// copy data in needle
						foreach ($setting_value as $value)
						{
							/** copy email templates if need */
							$check = EmailTemplates::model()->findByAttributes(array('lang_code' => $value));

							if (empty($check))
							{
								$templates = EmailTemplates::model()->findAll(array('group' => 'template_code'));

								/** @var EmailTemplates $template */
								foreach ($templates as $template)
								{
									$attributes = $template->attributes;
									$attributes['lang_code'] = $value;
									$new_template = new EmailTemplates();
									$new_template->setAttributes($attributes);
									$new_template->save();
								}
							}

							/** copy ticket categories if need */
							$ticket_categories  = TicketCategory::model()->findAll();

							/** @var TicketCategory $category */
							foreach ($ticket_categories as $category)
							{
								$translations = CJSON::decode($category->translations);

								if (empty($translations[$value]))
								{
									$translations[$value] = reset($translations);
									$category->translations = CJSON::encode($translations);
									$category->save();
								}
							}
						}
					}
				}
				else if ($setting_label == 'helpdesk_url' && !empty($setting_value))
				{
					$parse = parse_url($setting_value);
					$setting->value = $parse['host'];
				}
				else
					$setting->value = $setting_value;

				$setting->save();
			}

			Yii::import('application.vendors.*');
			include_once 'MailParser/MailParser.class.php';
			new MailParser();

			Yii::app()->user->setFlash('success', _t('changes saved'));

			$this->redirect($this->createUrl('/settings/index', array('section' => $_REQUEST['section'])));
		}

		$data = $languages = array();

		if (!empty($_REQUEST['section']))
		{
			$criteria = new CDbCriteria();
			$criteria->condition = 'group_id = :group_id AND visible = 1';
			$criteria->params[':group_id'] = $_REQUEST['section'];

			$dataProvider=new CActiveDataProvider('Settings', array ('criteria' => $criteria, 'pagination' => array('pageSize' => 100)));
			/**
			 * @var Settings $setting
			 */
			foreach ($dataProvider->getData() as $setting)
			{
				$data['settings'][$setting->group->id]['values'][] = $setting;
				$data['settings'][$setting->group->id]['name'] = $setting->group->name;
			}

			if ($_REQUEST['section'] == 1)
			{
				$dirs = scandir(dirname(__FILE__) . '/../messages');

				foreach ($dirs as $dir)
				{
					if (!in_array($dir, array('.', '..')))
						$languages[$dir] = $dir;
				}
			}
		}

		$accounts = array ();

		if ($_REQUEST['section'] == 'email_accounts')
		{
			$new = new EmailAccounts();
			$redirect = false;

			if (!empty($_REQUEST['addresses']))
			{
				if (!empty($_REQUEST['addresses']['new']) && !empty($_REQUEST['addresses']['new']['email_address']))
				{
					$new->attributes = $_REQUEST['addresses']['new'];
					$new->email_address = $_REQUEST['addresses']['new']['email_address'];

					$res = $new->save();
					$errors = $new->getErrors();

					if (!empty($errors))
					{
						foreach ($new->getErrors() as $error)
							Yii::app()->user->setFlash('error', $error[0]);
					}

					$redirect = empty($errors) ? true : false;
				}
			}

			$accounts[] = $new;

			/** @var EmailAccounts[] $other */
			$other = EmailAccounts::model()->findAll();

			foreach ($other as $account)
			{
				if (!empty($_REQUEST['addresses']) && !empty($_REQUEST['addresses'][$account->id]))
				{
					$account->attributes = $_REQUEST['addresses'][$account->id];
					$account->save();
					$redirect = true;
				}

				$accounts[] = $account;
			}

			if (!empty($_REQUEST['delete']))
			{
				EmailAccounts::model()->deleteByPk($_REQUEST['delete_id']);
				$redirect = true;
			}

			if ($redirect)
				$this->redirect($this->createUrl('/settings/index', array ('section' => 'email_accounts')));
		}

		if (in_array($_REQUEST['section'], array('spam_filters', 'email_accounts')) && $this->disabledFeature())
		{
			throw new CHttpException(404, _t('disabled in free'));
		}

		if (Settings::get('facebook_token') && $_REQUEST['section'] == 4)
		{
			$fb = FacebookApi::getConnection();

			$response = $fb->get(
				'/me?fields=accounts,name,email'
			);

			$graphObject = $response->getDecodedBody();

			if (!empty($_REQUEST['facebook_page']))
			{
				foreach ($graphObject['accounts']['data'] as $page)
				{
					if ($page['id'] == $_REQUEST['facebook_page'])
						Settings::set('facebook_page_token', $page['access_token']);
				}

				Settings::set('facebook_page', $_REQUEST['facebook_page']);
				Yii::app()->user->setFlash('success', _t('facebook page saved', ['{page}' => $page['name']]));
				$this->redirect($this->createUrl('/settings/index', array ('section' => '4')));
			}

			if (!empty($graphObject['accounts']['data']))
				$data = $graphObject['accounts']['data'];
		}

		$this->render('index',array(
			'data' 		=> $data,
			'section'   => $_REQUEST['section'],
			'languages' => $languages,
			'accounts'  => $accounts,
		));
	}

	/**
	 * Manages all models.
	 */
	public function actionAdmin()
	{
		$model=new Settings('search');
		$model->unsetAttributes();  // clear any default values
		if(isset($_GET['Settings']))
			$model->attributes=$_GET['Settings'];

		$this->render('admin',array(
			'model'=>$model,
		));
	}

	/**
	 * Returns the data model based on the primary key given in the GET variable.
	 * If the data model is not found, an HTTP exception will be raised.
	 * @param integer the ID of the model to be loaded
	 */
	public function loadModel($id)
	{
		$model=Settings::model()->findByPk($id);
		if($model===null)
			throw new CHttpException(404, _t('page does not exist'));
		return $model;
	}

	/**
	 * Performs the AJAX validation.
	 * @param CModel the model to be validated
	 */
	protected function performAjaxValidation($model)
	{
		if(isset($_POST['ajax']) && $_POST['ajax']==='settings-form')
		{
			echo CActiveForm::validate($model);
			Yii::app()->end();
		}
	}

	public function actionPriorities()
	{
		if (!empty($_REQUEST['priority_id']))
		{
			$ticket_priority = TicketPriority::model()->findByPk($_REQUEST['priority_id']);

			if (!empty($ticket_priority))
			{
				$sql = 'UPDATE ' . Ticket::model()->tableName() . ' SET priority_id = 0 WHERE priority_id = :id';
				$command = Yii::app()->db->createCommand($sql);
				$command->bindParam('id', $_REQUEST['priority_id']);
				$command->execute();

				$ticket_priority->delete();
			}
		}

		if (!empty($_POST['priorities']))
		{
			foreach ($_POST['priorities'] as $id => $name)
			{
				if (empty($id))
					continue;

				if ($id != 'new')
				{
					$ticket_priority = TicketPriority::model()->findByPk($id);
					if (!empty($ticket_priority))
					{
						$ticket_priority->name = $name;
						$ticket_priority->save(false);
					}
				}
				else if (!empty($name))
				{
					$priority = new TicketPriority();
					$priority->name = $name;
					$priority->save(false);
				}
			}
		}

		$this->redirect($this->createUrl('/settings/index', array ('section' => 'priorities')));
	}

	public function actionCategories()
	{
		if (isset($_REQUEST['category_id']))
		{
			/**
			 * @var TicketCategory $ticket_category
			 */
			$ticket_category = TicketCategory::model()->findByPk($_REQUEST['category_id']);

			if (!empty($ticket_category) && !$ticket_category->default_category)
			{
				/**
				 * @var TicketCategory $default
				 */
				$default = TicketCategory::model()->findByAttributes(array('default_category' => 1));
				$sql = 'UPDATE ' . Ticket::model()->tableName() . ' SET category_id = ' . $default->id . ' WHERE category_id = :id';
				$command = Yii::app()->db->createCommand($sql);
				$command->bindParam('id', $_REQUEST['category_id']);
				$command->execute();

				$ticket_category->delete();
			}
		}

		if (!empty($_POST['categories']))
		{
			foreach ($_POST['categories'] as $id => $name)
			{
				if (empty($id))
					continue;

				$category = false;

				if ($id !== 'new')
					$category = TicketCategory::model()->findByPk($id);
				else if (!empty($name))
					$category = new TicketCategory();

				if (!empty($category))
				{
					$translations = CJSON::decode($category->translations);
					$translations[Yii::app()->language] = $name;

					$category->translations = CJSON::encode($translations);
					$category->save();
				}
			}
		}

		Yii::app()->user->setFlash('success', _t('changes saved'));

		$this->redirect($this->createUrl('/settings/index', array ('section' => 'categories')));
	}

	public function actionFilters()
	{
		if (isset($_REQUEST['filter_id']))
		{
			/** @var SpamFilters $filter */
			$filter = SpamFilters::model()->findByPk($_REQUEST['filter_id']);

			if (!empty($filter))
				$filter->delete();
		}

		if (!empty($_POST['spam_filters']))
		{
			foreach ($_POST['spam_filters'] as $id => $value)
			{
				if (empty($id))
					continue;

				if ($id != 'new')
				{
					$filter = SpamFilters::model()->findByPk($id);

					if (!empty($filter))
					{
						$filter->setAttributes($value);
						$filter->save();
					}
				}
				else if (!empty($value['value']))
				{
					$filter = new SpamFilters();
					$filter->setAttributes($value);
					$filter->save();
				}
			}
		}

		Yii::app()->user->setFlash('success', _t('changes saved'));

		$this->redirect($this->createUrl('/settings/index', array ('section' => 'spam_filters')));
	}

	public function actionEmailTemplates()
	{
		foreach ($_REQUEST['templates'] as $template_code => $template_data)
		{
			/** @var EmailTemplates $template */
			$template = EmailTemplates::model()->findByPk(array('template_code' => $template_code, 'lang_code' => Yii::app()->language, 'account_id' => $template_data['account_id']));
			$template->setAttributes($template_data);
			$template->save();
		}

		Yii::app()->user->setFlash('success', _t('changes saved'));

		$this->redirect($this->createUrl('/settings/index', array ('section' => 'email_templates')));
	}

	public function actionEmailAccounts()
	{
		$accounts[] = new EmailAccounts();

		if (!empty($_REQUEST['addresses']))
		{
			$addresses = $_REQUEST['addresses'];

			if (!empty($addresses['new']) && !empty($addresses['new']['email_address']))
			{
				$accounts[0]->attributes = $addresses['new'];
				$accounts[0]->save();
			}
		}

		$this->redirect($this->createUrl('/settings/index', array ('section' => 'emailAccounts')));
	}

	public function actionCheckEmail()
	{
		Yii::import('application.vendors.*');

		$_settings = empty($_REQUEST['requested_id']) ? $_REQUEST['settings'] : $_REQUEST['addresses'][$_REQUEST['requested_id']];

		$msg = _t('email settings works');

		$imap = new EIMap(
			'{' . $_settings['email_host'] . ':' . $_settings['email_port'] . '/imap/ssl}INBOX',
			$_settings['email_address'],
			$_settings['email_password']
		);

		$imap->connect();

		// check SMTP
		try {
			Yii::app()->mail->transportOptions = array (
				'host'		 => $_settings['smtp_host'],
				'username'	 => $_settings['email_address'],
				'password'	 => $_settings['email_password'],
				'port'		 => $_settings['smtp_port'],
				'encryption' => $_settings['smtp_encryption'],
				'timeout'	 => 10,
			);

			Yii::app()->mail->transportType = 'smtp';

			$message = new YiiMailMessage();
			$message->addTo('check-email@re-desk.com');
			$message->addFrom($_settings['email_address']);
			$message->setBody('Test body');

			@Yii::app()->mail->send($message);
		} catch (Exception $e) {
			echo $e->getMessage();
			exit;
		}

		echo $msg;
		Yii::app()->end();
	}

	public function sortSettings($pattern, $settings)
	{
		$return = array ();

		foreach ($pattern as $key)
		{
			foreach ($settings as $k => $setting)
			{
				if ($setting->label == $key)
				{
					$return[] = $setting;
					unset($settings[$k]);
				}
			}
		}

		if (!empty($settings))
		{
			foreach ($settings as &$setting)
				$return[] = $setting;
		}

		return $return;
	}

	/*public function actionTwitter()
	{
		$oauth_nonce = md5(uniqid(rand(), true));
		$oauth_timestamp = time();

		$oauth_base_text = "GET&" .
			urlencode('https://api.twitter.com/oauth/request_token') . "&" .
			urlencode(
				"oauth_callback=" . urlencode($this->createAbsoluteUrl('/settings/twitterCallback')) . "&" .
				"oauth_consumer_key=" . "&" .
				"oauth_nonce=" . $oauth_nonce . "&" .
				"oauth_signature_method=HMAC-SHA1&" .
				"oauth_timestamp=" . $oauth_timestamp . "&" .
				"oauth_version=1.0"
			);

		$key = '' . "&";
		$oauth_signature = self::_encode($oauth_base_text, $key);

		$url = 'https://api.twitter.com/oauth/request_token' .
			'?oauth_callback=' . urlencode($this->createAbsoluteUrl('/settings/twitterCallback')) .
			'&oauth_consumer_key=' .
			'&oauth_nonce=' . $oauth_nonce .
			'&oauth_signature=' . urlencode($oauth_signature) .
			'&oauth_signature_method=HMAC-SHA1' .
			'&oauth_timestamp=' . $oauth_timestamp .
			'&oauth_version=1.0';

		if (!($response = @file_get_contents($url))) {
			return false;
		}

		parse_str($response, $result);

		if (empty($result['oauth_token_secret'])) {
			return false;
		}

		$_SESSION['oauth_token_secret'] = $result['oauth_token_secret'];

		$this->redirect('https://api.twitter.com/oauth/authorize' . '?oauth_token=' . $result['oauth_token']);
	}

	public function actiontwitterCallback()
	{
		if (!empty($_REQUEST['oauth_token']) && !empty($_REQUEST['oauth_verifier']))
		{
			// request access token
			$oauth_nonce = md5(uniqid(rand(), true));
			$oauth_timestamp = time();
			$oauth_token_secret = $_SESSION['oauth_token_secret'];

			$oauth_base_text = "GET&" .
				urlencode('https://api.twitter.com/oauth/access_token') . "&" .
				urlencode(
					"oauth_consumer_key=" . Settings::get('consumer_key') . "&" .
					"oauth_nonce=" . $oauth_nonce . "&" .
					"oauth_signature_method=HMAC-SHA1&" .
					"oauth_token=" . $_REQUEST['oauth_token'] . "&" .
					"oauth_timestamp=" . $oauth_timestamp . "&" .
					"oauth_verifier=" . $_REQUEST['oauth_verifier'] . "&" .
					"oauth_version=1.0"
				);

			$key = Settings::get('consumer_secret') . "&" . $oauth_token_secret;
			$oauth_signature = self::_encode($oauth_base_text, $key);

			$url = 'https://api.twitter.com/oauth/access_token' .
				'?oauth_consumer_key=' . Settings::get('consumer_key') .
				'&oauth_nonce=' . $oauth_nonce .
				'&oauth_signature_method=HMAC-SHA1' .
				'&oauth_token=' . urlencode($_REQUEST['oauth_token']) .
				'&oauth_timestamp=' . $oauth_timestamp .
				'&oauth_verifier=' . urlencode($_REQUEST['oauth_verifier']) .
				'&oauth_signature=' . urlencode($oauth_signature) .
				'&oauth_version=1.0';

			if (!($response = @file_get_contents($url))) {
				return false;
			}

			parse_str($response, $result);

			if (empty($result['oauth_token']) || empty($result['user_id'])) {
				return false;
			}

			Settings::set('oauth_token_secret', $result['oauth_token_secret']);
			Settings::set('oauth_token', $result['oauth_token']);
			Settings::set('twitter_account', $result['screen_name']);

			$this->redirect('/settings/index?section=3');
		}
		else
			return false;
		// @TODO send notification about wrong request
	}

	public function actionDeleteTwitter()
	{
		Settings::set('oauth_token', '');
		Settings::set('oauth_token_secret', '');

		$this->redirect('/settings/index?section=3');
	}

	public function actionFacebook()
	{
		$fb_token = Settings::get('facebook_token');

		if (empty($fb_token) && empty($_REQUEST['code']))
		{
			$facebook = FacebookApi::getConnection();

			$need_permissions = array (
				'manage_pages',
                'read_page_mailboxes',
				'publish_pages',
			);

			$helper = $facebook->getRedirectLoginHelper();

			$loginUrl = $helper->getLoginUrl(str_replace(':443', '', Yii::app()->createAbsoluteUrl('/settings/facebook/', array('section' => 4))), $need_permissions);

			header('Location: ' . $loginUrl);

			exit;
		}
		// get pages list
		else if (!empty($_REQUEST['code']))
		{
			$fb = FacebookApi::getConnection();

			$helper = $fb->getRedirectLoginHelper();

			try {
				$accessToken = $helper->getAccessToken();
			} catch(Facebook\Exceptions\FacebookResponseException $e) {
				// When Graph returns an error
				echo 'Graph returned an error: ' . $e->getMessage();
				exit;
			} catch(Facebook\Exceptions\FacebookSDKException $e) {
				// When validation fails or other local issues
				echo 'Facebook SDK returned an error: ' . $e->getMessage();
				exit;
			}

			$oAuth2Client = $fb->getOAuth2Client();

			if (!$accessToken->isLongLived())
				$accessToken = $oAuth2Client->getLongLivedAccessToken($accessToken);

			Settings::set('facebook_token', $accessToken->getValue());

			$this->redirect('/settings/facebook');
		}
		else
			$this->redirect('/settings/index?section=4');
	}

	public function actionDisconnect_Facebook()
	{
		Settings::set('facebook_token', '');
		Settings::set('facebook_page', '');
		Settings::set('facebook_page_token', '');

		$this->redirect('/settings/index?section=4');
	}*/


	private static function _encode($string, $key) {
		return base64_encode(hash_hmac("sha1", $string, $key, true));
	}
}
