<?php

require_once __DIR__ . '/../vendors/twitterOAuth/TwitterOAuth.php';
require_once __DIR__ . '/../vendors/twitterOAuth/Exception/TwitterException.php';

use TwitterOAuth\TwitterOAuth;

class TicketController extends HDController
{
	/**
	 * @var string the default layout for the views. Defaults to '//layouts/column2', meaning
	 * using two-column layout. See 'protected/views/layouts/column2.php'.
	 */
	public $layout='//layouts/column2';
	public $load_more = true;
	public $page;

	public $show_actions = true;

	public $ticket_selected = true;
	public $select_all = false;
	public $is_twitter = false;

	public function allowedActions()
	{
		return $this->user ? ('favorite, index, close, changeStatus, group, unread, captcha' . ($this->user->is_worker ? ', assign, note, deleteMessage' : '')) : 'captcha, close';
	}

	/**
	 * @return array action filters
	 */
	public function filters()
	{
		return array(
			'rights'
		);
	}

	public function accessRules() {
		return array(
			array(
				'allow', 'actions' => array('favorite'), 'users' => array('@')
			)
		);
	}

	public function actions(){
		return array(
			'captcha'=>array(
				'class'=>'CCaptchaAction',
			),
		);
	}

	/**
	 * Displays a particular model.
	 * @param integer $id the ID of the model to be displayed
	 */
	public function actionView($id)
	{
		/**
		 * @var Ticket $ticket
		 */
		$ticket = $this->loadModel($id);

		$message = $this->newMessage($ticket);
		if (!empty($message->id))
		{
			$params = array('view','id'=>$ticket->id);
			if (!empty($_REQUEST['access_key']))
				$params['access_key'] = $_REQUEST['access_key'];

			$params['has_sent'] = true;

			$this->redirect($params);
		}

		if ($this->user)
			$ticket->setView($this->user->id);

		$this->header_content = $this->renderPartial('_header_view', array('ticket' => $ticket), true);

		$this->render('view',array(
			'model'         => $ticket,
			'message'       => $message,
			'categories'    => TicketCategory::model()->findAll(),
		));
	}

	protected function newMessage(Ticket $ticket)
	{
		$message = new TicketMessage();

		if ($_REQUEST['new_message'])
			$_POST['TicketMessage'] = $_SESSION['facebook_message'];

		if (!empty($_POST['TicketMessage']))
		{
			$message->attributes = $_POST['TicketMessage'];

			if ($message_id = $ticket->addMessage($message))
			{
                $message->addAttachments();
				Yii::app()->user->setFlash('success', _t('message sent'));
			}
		}

		$message->ticket_id = $ticket->id;

		return $message;
	}

	/**
	 * Creates a new model.
	 * If creation is successful, the browser will be redirected to the 'view' page.
	 * @var int $message_id
	 */
	public function actionCreate($id = 0)
	{
		$model=new Ticket;
		$this->header_content = $this->renderPartial('_header_view', array('ticket' => $model), true);;

		if (!empty($this->user))
		{
			$model->author_name = $this->user->is_worker ? '' : $this->user->profile->full_name;
			$model->author_email = $this->user->is_worker ? '' : $this->user->email;

			if (!$this->user->is_worker)
				$model->user_id = $this->user->id;
		}

		// Uncomment the following line if AJAX validation is needed
		$this->performAjaxValidation($model);

		if (isset($_POST['Ticket']))
		{
			$model->attributes=$_POST['Ticket'];

			if ($model->save())
			{
				$params = array('view','id'=>$model->id);

				// ticket from guest
				if ($model->access_key)
					$params['access_key'] = $model->access_key;

				// create first message
				$message = new TicketMessage();

				if ($this->user && $this->user->is_worker)
				{
					$message->author_email = $this->user->email;
					$message->author_name = $this->user->profile->full_name;
				}
				else
				{
					$message->author_email = $model->author_email;
					$message->author_name = $model->author_name;
				}

				$message->message = $model->content;
				$message->ticket_id = $model->id;
				$message->moved_from = empty($_POST['message_id']) ? 0 : $_POST['message_id'];
				$message->save(false);

				$message->addAttachments();

				if (!empty($_POST['message_id']))
				{
					/**
					 * @var TicketMessage $from_message
					 */
					$from_message = TicketMessage::model()->findByPk($_POST['message_id']);
					if (!empty($from_message))
					{
						$from_message->moved_to = $message->id;
						$from_message->save();
					}
				}

				$model->sendNotification($model->author_email, $model->author_name);

				$this->redirect($params);
			}
		}

		$message = null;

		if ($id)
		{
			/**
			 * @var TicketMessage $message
			 */
			$message = TicketMessage::model()->findByPk($id);
			$model->author_name = $message->author_name;
			$model->author_email = $message->author_email;
			$model->content = $message->message;
		}

		$this->render('create',array(
			'model' => $model,
			'id' => $id,
		));
	}

	/**
	 * Updates a particular model.
	 * If update is successful, the browser will be redirected to the 'view' page.
	 * @param integer $id the ID of the model to be updated
	 */
	public function actionUpdate($id)
	{
		$model=$this->loadModel($id);

		if(isset($_POST['Ticket']))
		{
			$model->attributes=$_POST['Ticket'];
			// wtf?
			$model->category_id = $_POST['Ticket']['category_id'];

			if($model->save())
				$this->redirect(array('view','id'=>$model->id));
		}

		$this->render('update',array(
			'model'=>$model,
		));
	}

	/**
	 * Deletes a particular model.
	 * If deletion is successful, the browser will be redirected to the 'admin' page.
	 * @param integer $id the ID of the model to be deleted
	 */
	public function actionDelete($id)
	{
		$this->loadModel($id)->delete();
		$this->redirect(array('ticket/index'));
	}

	public function actionChangeCategory($id)
	{
		if (Yii::app()->request->isAjaxRequest)
		{
			// update message
			if (isset($_POST['item']))
			{
				$model = $this->loadModel($id);
				$model->category_id = $_POST['item'];
				$model->save(false);

				$this->renderPartial(
					'_category_input',
					array (
						'data'    => $model,
						'manager' => $this->manager,
						'user_id' => $this->user->id,
						'categories' => TicketCategory::model()->findAll(),
					)
				);
			}

			Yii::app()->end();
		}
	}

	public function actionChangeStatus($id)
	{
		if (Yii::app()->request->isAjaxRequest)
		{
			// update message
			if (isset($_POST['item']))
			{
				$model = $this->loadModel($id);
				$model->is_open = $_POST['item'] == 'open' ? 1 : 0;
				$model->save(false);

				$message = $this->newMessage($model);

				$this->ajax_updates = array (
					'status_input'  => $this->renderPartial('_status_input', array ('data' => $model), true),
					'status_block'  => $this->renderPartial('_ticket_status_block', array ('data' => $model), true),
				 	'reply_section' => $this->renderPartial('_reply_block', array ('data' => $model, 'message' => $message), true)
				);
			}

			Yii::app()->end();
		}
	}

	/**
	 * Lists all models.
	 */
	public function actionIndex()
	{
		if (!Settings::get('email_address'))
			Yii::app()->user->setFlash('error', _t('please configure email'));

		$this->show_select = $this->show_selected = $this->show_refresh = true;
		$this->pageTitle = (_t('tickets') . ($this->total_found ? " ($this->total_found)" : ''));

		if ($this->user && $_REQUEST['sub_action'] == 'clean' && $_REQUEST['folder'] == TicketFolders::TRASH && $this->manager->hasCredential($this->user->id, 'Ticket.Delete'))
		{
			Ticket::model()->deleteAllByAttributes(array ('folder_id' => TicketFolders::TRASH));
			$this->redirect($this->createUrl('ticket/index', array('folder' => TicketFolders::TRASH)));
		}

		if (empty($_REQUEST['filter']) && $_REQUEST['folder'] != TicketFolders::TRASH)
		{
			$this->select_all = true;
			$_REQUEST['filter'] = '';
		}

		$criteria = Ticket::getBaseCriteria($_REQUEST['filter'], $_REQUEST['selector'], $_REQUEST['q'], $_REQUEST['folder']);

		if ($this->user)
		{
			if ($this->user->is_worker)
				$criteria->order = 'last_client DESC';
			else
				$criteria->order = 'last_staff DESC';
		}
		else
			$criteria->order = 'created_at DESC';

		$per_page = 50;

		$dataProvider=new CActiveDataProvider('Ticket',
			array (
				'criteria' => $criteria,
				'pagination' => array(
					'pageSize' => intval($per_page),
					'pageVar' => 'page',
					'validateCurrentPage' => false
				)
			)
		);

		$this->per_page = $per_page;
		$this->page = $_REQUEST['page'] ? $_REQUEST['page'] : 1;
		$this->total_list_found = $dataProvider->totalItemCount;

		$this->load_more = $this->total_list_found > $this->page * $per_page;
		$this->show_actions = true;

		$tickets = $dataProvider->getData();
		$ticket_ids = array ();

		if (!empty($tickets))
		{
			foreach ($tickets as $ticket)
				/** @var Ticket $ticket */
				$ticket_ids[] = $ticket->id;
		}

		// scoped data - simple caching
		Ticket::model()->setScopeMessages($ticket_ids);
		Ticket::model()->setScopeFavorites($ticket_ids);
		Ticket::model()->setScopeViewed($ticket_ids);

		if (Yii::app()->request->isAjaxRequest)
		{
			$data = $dataProvider->getData();

			if (empty($data) && !empty($_REQUEST['page']))
				return '';

			$this->ajax_updates = [
				'content' 	  => $this->renderPartial('index', array('dataProvider' => $dataProvider), true),
			];

			Yii::app()->end();
		}
		else
		{
			return $this->render('index',array(
				'dataProvider' => $dataProvider,
			));
		}
	}

	public function actionClose($id)
	{
		$ticket = $this->loadModel($id);

		if ($ticket)
		{
			if (!$this->user && $ticket->access_key != $_REQUEST['access_key'])
				Yii::app()->end();

			$ticket->is_open = !$ticket->is_open;
			$ticket->save(false);

			$message = $this->newMessage($ticket);

			Yii::app()->user->setFlash('success', _t('status changed'));

			$this->ajax_updates = array (
				'status_block'  => $this->renderPartial('_ticket_status_block', array ('data' => $ticket), true),
				'reply_section' => $this->renderPartial('_reply_block', array ('data' => $ticket, 'message' => $message), true),
				'status_input'  => $this->renderPartial('_status_input', array ('data' => $ticket), true)
			);

			Yii::app()->end();
		}
	}

	/**
	 * Manages all models.
	 */
	public function actionAdmin()
	{
		$model=new Ticket('search');
		$model->unsetAttributes();  // clear any default values
		if(isset($_GET['Ticket']))
			$model->attributes=$_GET['Ticket'];

		$this->render('admin',array(
			'model'=>$model,
		));
	}

	public function actionNote($id)
	{
		$ticket = $this->loadModel($id);

		$_REQUEST['note'] = trim($_REQUEST['note']);

		if ($ticket)
		{
			$ticket->note = $_REQUEST['note'];
			$ticket->save(false);
		}

		echo nl2br($_REQUEST['note']);

		Yii::app()->end();
	}

	/**
	 * Returns the data model based on the primary key given in the GET variable.
	 * If the data model is not found, an HTTP exception will be raised.
	 * @param integer the ID of the model to be loaded
	 * @return Ticket $model
	 */
	public function loadModel($id)
	{
		$model=Ticket::model()->findByPk($id);
		if($model===null)
			throw new CHttpException(404, _t('page does not exist'));
		return $model;
	}

	/**
	 * Performs the AJAX validation.
	 * @param CModel $model the model to be validated
	 */
	protected function performAjaxValidation($model)
	{
		if(isset($_POST['ajax']) && $_POST['ajax']==='ticket-form')
		{
			echo CActiveForm::validate($model);
			Yii::app()->end();
		}
	}

	function checkGuestAccess()
	{
		/** @var User $user */
		$user = Yii::app()->getModule('user')->user();
		/** @var $ticket Ticket */
		$ticket = Ticket::model()->findByPk($_REQUEST['id']);

		if (!$ticket)
			return false;

		if ($user)
			return $ticket->canAccess($user);
		else
		{
			if (empty($_REQUEST['access_key']))
				return false;

			if ($ticket)
				return $ticket->access_key == $_REQUEST['access_key'];
		}
	}

	public function actionFavorite($id)
	{
		$ticket = $this->loadModel($id);

		if (!empty($ticket) && !empty($this->user))
		{
			$exists = TicketFavorite::model()->findByAttributes(array('ticket_id' => $ticket->id, 'user_id' => $this->user->id));

			Yii::app()->cache->delete(Ticket::CACHE_STAR . $id . '_' . $this->user->id);

			if (empty($exists))
				$ticket->setStar($this->user->id);
			else
				$exists->delete();

			$this->ajax_updates = array (
				'favorite_' . $ticket->id => trim($this->renderPartial('_favorite_update', array ('model' => $ticket), true)),
				'star_buttons'            => $ticket->isStarred() ? _t('remove star') : _t('add star'),
			);
		}

		Yii::app()->end();
	}

	public function actionGroup()
	{
		if (!empty($_REQUEST['ids']) && !empty($this->user))
		{

			$criteria = new CDbCriteria();
			$criteria->addInCondition("id", $_POST['ids']);

			/**
			 * @var Ticket[] $tickets
			 */
			$tickets = Ticket::model()->findAll($criteria);

			foreach ($tickets as $ticket)
			{
				if ($_REQUEST['mode'] == 'unread' && $this->user->is_worker)
					$ticket->setUnread();
				elseif ($_REQUEST['mode'] == 'read')
					$ticket->setViews();
				elseif ($_REQUEST['mode'] == 'star')
					$ticket->setStar($this->user->id);
				elseif ($_REQUEST['mode'] == 'delete' && $this->manager->hasCredential($this->user->id, 'Ticket.Delete'))
					$ticket->delete();
				elseif ($_REQUEST['mode'] == 'inbox' && $this->manager->hasCredential($this->user->id, 'Ticket.Delete'))
				{
					$ticket->folder_id = TicketFolders::INBOX;
					$ticket->save(false);
				}
				elseif ($_REQUEST['mode'] == 'closed')
				{
					$ticket->is_open = false;
					$ticket->setViews();
					$ticket->save(false);
				}
				elseif ($_REQUEST['mode'] == 'open')
				{
					$ticket->is_open = true;
					$ticket->save(false);
				}
			}
		}

		Yii::app()->end();
	}

	public function actionUnread($id)
	{
		$ticket = $this->loadModel($id);
		if (!empty($ticket))
			$ticket->setUnread();

		$this->redirect($this->createUrl('ticket/index'));
	}

	public function actionAssign($id)
	{
		if (Yii::app()->request->isAjaxRequest)
		{
			// update message
			if (!empty($_REQUEST['item']))
			{
				$ticket = $this->loadModel($id);
				$ticket->assigned_id = $_REQUEST['item'];
				$ticket->save(false);

				$ticket->sendAssignNotification();

				echo $this->renderPartial('_user_input', array ('model' => $ticket));
			}

			Yii::app()->end();
		}
	}

	public function actionDeleteMessage($id)
	{
		/** @var TicketMessage $message */
		$message = TicketMessage::model()->findByPk($id);
		$ticket_id = $message->ticket->id;

		if (!empty($message) && $this->manager->hasCredential($this->user->id, 'Ticket.Delete'))
			$message->delete();

		$this->redirect($this->createUrl('ticket/view', array ('id' => $ticket_id)));
	}

	public function getCategoryHtml(Ticket $ticket)
	{
		static $categories;

		if (!$categories)
			$categories = TicketCategory::model()->findAll();

		/** @var AppDbAuthManager $manager */
		$manager = Yii::app()->getAuthManager();

		if ($manager->hasCredential($this->user->id, 'Ticket.ChangeCategory'))
			$category_extra_html = '<div id="category_' .  $ticket->id . '"><div class="dropdown hd-category-' . $ticket->id . '">' . $this->renderPartial(
				'_category_input',
				array (
					'data'       => $ticket,
					'manager'    => $manager,
					'user_id'    => $this->user->id,
					'categories' => $categories,
				),
				true
			) . '</div></div>';
		else
			$category_extra_html = $ticket->category->lang_name;

		return $category_extra_html;
	}

	function createTicket($params)
	{
		$body = $params['body'];
		// TODO: print log information

		if (!$params['subject'] && !$body) {
			return null;
		}

		$ticket = new Ticket();
		$ticket->subject = $params['subject'];
		$ticket->author_email = $params['author_email'];
		$ticket->author_name = $params['author_name'];
		$ticket->user_id = 0;
		$ticket->is_open = 1;
		$ticket->content = $body;

		if (!empty($params['created_at']))
			$ticket->created_at = $params['created_at'];

		if (!empty($params['folder']))
			$ticket->folder_id = $params['folder'];

		if (!empty($account) && $account->department)
		{
			$category = TicketCategory::model()->findByPk($account->department);

			if (!empty($category))
				$ticket->category_id = $account->department;
		}

		/**
		 * @var User $user
		 */
		$user = User::model()->findByAttributes(array('email' => $params['author_email']));

		if (!empty($user))
			$ticket->user_id = $user->id;

		$ticket->save(false);

		// next - create message
		$message = new TicketMessage();
		$message->author_email = $params['author_email'];
		$message->author_name = $params['author_name'];
		$message->message = $body;
		$message->ticket_id = $ticket->id;
		$message->user_id = $ticket->user_id;

		if ($params['external_id'])
		{
			$message->external_id = $params['external_id'];
			$message->external_type = $params['external_type'];
		}

		if (!empty($params['created_at']))
			$message->created_at = $params['created_at'];

		$message->save(false);

		if (!empty($params['attachments']))
		{
			foreach ($params['attachments'] as $attach)
			{
				$attach_add = new MessageAttachment();
				$attach_add->file_name = $attach['file_name'];
				$attach_add->file_type = $attach['mime_type'];
				$attach_add->content = $attach['content'];
				$attach_add->message_id = $message->id;
				$attach_add->size = $attach['size'];
				$attach_add->save(false);
			}
		}

		if ($ticket->folder_id != TicketFolders::TWITTER)
			$ticket->sendNotification($params['author_email'], $params['author_name'], $params['external_from_address'] ? $params['external_from_address'] : '', $params['external_from_name'] ? $params['external_from_name'] : '');
	}

	protected function createMessage($params)
	{
		if (!$params['body'] && count($params['attachments']) == 0) {
			return null;
		}

		/**
		 * @var Ticket $ticket
		 */
		$ticket = Ticket::model()->findByAttributes(array('tracking_id' => $params['ticket_id']));

		if (!$ticket) {
			//TODO: send notification about absence of the ticket
			return false;
		}

		$assigned_id = 0;

		foreach ($ticket->messages as $message)
		{
			if ($message->assign_id)
			{
				$assigned_id = $message->assign_id;
				break;
			}
		}

		$message = new TicketMessage();
		$message->author_email = $params['author_email'];
		$message->author_name = $params['author_name'];
		$message->message = $params['body'];
		$message->ticket_id = $ticket->id;
		$message->assign_id = $assigned_id;

		if ($params['external_id'])
		{
			$message->external_id = $params['external_id'];
			$message->external_type = $params['external_type'];
		}

		/**
		 * @var User $user
		 */
		$user = User::model()->findByAttributes(array('email' => $params['author_email']));
		if (!empty($user))
			$message->user_id = $user->id;

		$message->save();

		if (!empty($params['attachments']))
		{
			foreach ($params['attachments'] as $attach) {
				$attach_add = new MessageAttachment();
				$attach_add->file_name = $attach['file_name'];
				$attach_add->file_type = $attach['mime_type'];
				$attach_add->size = mb_strlen($attach['content'], APP_ENCODING);
				$attach_add->content = $attach['content'];
				$attach_add->message_id = $message->id;
				$attach_add->save(false);
			}
		}

		$ticket = $message->ticket;
		$ticket->is_open = 1;
		$ticket->save(false);

		return true;
	}
}
