<?php
/***************************************************************************
 *                                                                          *
 *				   (c) 2013 Re:Desk Development Team    				    *
 *                                                                          *
 * This  is  commercial  software,  only  users  who have purchased a valid *
 * license  and  accept  to the terms of the  License Agreement can install *
 * and use this program.                                                    *
 *                                                                          *
 ****************************************************************************
 * PLEASE READ THE FULL TEXT  OF THE SOFTWARE  LICENSE   AGREEMENT  IN  THE *
 * "copyright.txt" FILE PROVIDED WITH THIS DISTRIBUTION PACKAGE.            *
 ****************************************************************************/

/**
 * This is the model class for table "tbl_ticket".
 *
 * The followings are the available columns in table 'tbl_ticket':
 * @property integer $id
 * @property string $subject
 * @property string $content
 * @property string $note
 * @property integer $user_id
 * @property string $tracking_id
 * @property integer $assigned_id
 * @property integer $status_id
 * @property integer $priority_id
 * @property integer $created_at
 * @property integer $edited_at
 * @property integer $category_id
 * @property string $access_key
 * @property string $author_email
 * @property string $author_name
 * @property integer $is_open
 * @property integer $last_staff
 * @property integer $last_client
 * @property integer $folder_id
 * @property string $external_id
 * @property string $external_type
 * @property string $external_updated_time
 * @property integer $account_id
 * @property string $order_id
 * @property integer $close_time
 * @property string $permalink
 *
 * The followings are the available model relations:
 * @property TicketCategory $category
 * @property TicketFolders $folder
 * @property TicketPriority $priority
 * @property TicketStatus $status
 * @property TicketMessage[] $messages
 * @property int $message_count
 * @property TicketFavorite[] $favorites
 * @property TicketViews[] $views
 */
class Ticket extends CActiveRecord
{
	const ACCESS_KEY_LENGTH = 16;

	const MAX_NOTIFY_TIME = 180;

	const CACHE_STAR = 'star_';
	const CACHE_ORDERS = 'orders_';

	public $verifyCode;

	static $messagesCount;
	static $scopeFavorites;
	static $scopeViewed;
	static $usersInfo;

	/**
	 * Returns the static model of the specified AR class.
	 * @param string $className active record class name.
	 * @return Ticket the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}

	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return 'tbl_ticket';
	}

	public function beforeSave()
	{
		if ($this->isNewRecord)
		{
			if (!defined('CONSOLE_APP'))
			{
				/**
				 * @var User $user
				 */
				$user = Yii::app()->getModule('user')->user();
				$user_id = $user ? $user->id : 0;
			}
			else
				$user_id = 0;

			if ($user_id)
				$this->user_id = $user_id;

			$this->access_key = $this->getTracking(self::ACCESS_KEY_LENGTH);

			$status = TicketStatus::model()->findByAttributes(array('default' => 1));
			$category = TicketCategory::model()->findByAttributes(array('default_category' => 1));
			$priority = TicketPriority::model()->findByAttributes(array('default' => 1));

			$this->status_id = $status->id;
			$this->tracking_id = $this->getTracking();

			if (empty($this->category_id))
				$this->category_id = $category->id;

			/** @var UserCategories $assign */
			$assign = UserCategories::model()->findByAttributes(array('category_id' => $this->category_id));

			// ticket was assigned by category
			if (!empty($assign))
			{
				$this->assigned_id = $assign->user_id;
				//$this->sendAssignNotification();
			}
			else
			{
				$assign = User::model()->findByAttributes(array('is_worker' => 1));
				$this->assigned_id = $assign->id;
			}

			$this->priority_id = $priority->id;

			if (!$this->last_client)
				$this->last_client = time();

			if (!$this->last_staff)
				$this->last_staff = time();

			$this->is_open = 1;
		}

		if ($this->is_open != 1)
			$this->close_time = time();

		return parent::beforeSave();
	}

	protected function afterSave() {
		parent::afterSave();

		// Inform staff about new ticket
		if ($this->isNewRecord)
		{
			$send_emails = array ();

			/** @var User $user */
			foreach (User::model()->staff()->findAll('send_notifications = 1') as $user)
			{
				if ($this->canAccess($user) && !$this->stopNotification($user->email))
				{
					$send_emails[] = array (
						'email'	=> $user->email,
						'name'  => $user->profile->full_name,
					);
				}
			}

			if (!empty($send_emails))
			{
				/**
				 * @var YiiMailMessage $email
				 */
				$email = new YiiMailMessage();

				foreach ($send_emails as $email_addresses)
					$email->addTo($email_addresses['email'], $email_addresses['name']);

				$params = array (
					'{TICKET_SUBJECT}' => $this->subject,
					'{TICKET_ID}'      => $this->id,
					'{TICKET_AUTHOR}'  => $this->author_name,
					'{EMAIL_CONTENT}'  => TicketMessage::showEmail($this->content),
					'{TICKET_URL}'     => Yii::app()->createAbsoluteUrl('/ticket/view', array('id' => $this->id)),
					'{COMPANY_NAME}'   => Settings::get('company_name'),
				);

				$email->template_code = 'ticket_created';

				$email->setBody($params);

				$from_email = Settings::get('email_address');

				if (!$from_email)
					$from_email = Yii::app()->params['adminEmail'];
				else
					$from_email = array(Settings::get('email_address') => Settings::get('email_name'));

				// maybe this ticket from external email
				/** @var EmailAccounts $email_account */
				$email_account = EmailAccounts::model()->findByAttributes(array('department' => $this->category_id));
				if (!empty($email_account))
					$from_email = array ($email_account->email_address => $email_account->email_name);

				$email->setFrom($from_email);

				return sendAppMail($email, $email_account);
			}
		}
	}

	public function behaviors(){
		return array(
			'HDTimestampBehavior' => array(
				'class' => 'HDTimestampBehavior',
				'createAttribute' => 'created_at',
				'updateAttribute' => 'edited_at',
			)
		);
	}

	public function relations() {
		return array (
			'messages' => array (
				self::HAS_MANY,
				'TicketMessage',
				'ticket_id',
				'order' => 'messages.created_at ASC'
			),
			'message_count' => array (
				self::STAT,
				'TicketMessage',
				'ticket_id'
			),
			'status' => array (
				self::BELONGS_TO,
				'TicketStatus',
				'status_id'
			),
			'folder' => array(
				self::BELONGS_TO,
				'TicketFolders',
				'folder_id'
			),
			'category' => array (
				self::BELONGS_TO,
				'TicketCategory',
				'category_id'
			),
			'priority' => array (
				self::BELONGS_TO,
				'TicketPriority',
				'priority_id'
			),
			'favorites' => array(
				self::HAS_MANY,
				'TicketFavorite',
				'ticket_id'
			),
			'ticketViews' => array(
				self::HAS_MANY,
				'TicketViews',
				'ticket_id'
			),
		);
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		$rules = array (
			array('subject, author_email, author_name', 'required'),
			array('author_email', 'email', 'message' => _t('email incorrect')),
			array('user_id, assigned_id, category_id, account_id', 'numerical','integerOnly'=>true),
			array('created_at', 'default', 'value' => time()),
			array('edited_at', 'default', 'value' => time()),
			array('subject', 'length', 'max'=>255),
			array('content, order_id', 'safe'),
			array('tracking_id', 'length', 'max'=>10),
			array(
				'verifyCode',
				'captcha',
				'allowEmpty'=> defined('CONSOLE_APP') || !Yii::app()->user->isGuest || !CCaptcha::checkRequirements(),
			),
			// The following rule is used by search().
			// Please remove those attributes that should not be searched.
			array('id, subject, content, user_id, tracking_id, assigned_id, status_id, priority_id, created_at, edited_at', 'safe', 'on'=>'search'),
			array ('attachments', 'checkUpload'),
		);

		return $rules;
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		$user = Yii::app()->getModule('user')->user();

		if ($user && $user->is_worker)
		{
			$email = _t('client email');
			$name = _t('client name');
		}
		else
		{
			$email = _t('your email');
			$name = _t('your name');
		}

		return array(
			'id' => _t('id'),
			'subject' => _t('subject'),
			'content' => _t('content'),
			'user_id' => _t('author'),
			'tracking_id' => _t('tracking'),
			'assigned_id' => _t('assigned'),
			'status_id' => _t('status'),
			'priority_id' => _t('priority'),
			'created_at' => _t('created at'),
			'edited_at' => _t('edited at'),
			'category_id' => _t('department'),
			'author_email' => $email,
			'author_name' => $name,
			'verifyCode'  => _t('type verify code'),
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 * @return CActiveDataProvider the data provider that can return the models based on the search/filter conditions.
	 */
	public function search()
	{
		// Warning: Please modify the following code to remove attributes that
		// should not be searched.

		$criteria=new CDbCriteria;

		$criteria->compare('id',$this->id);
		$criteria->compare('subject',$this->subject,true);
		$criteria->compare('content',$this->content,true);
		$criteria->compare('user_id',$this->user_id);
		$criteria->compare('tracking_id',$this->tracking_id,true);
		$criteria->compare('assigned_id',$this->assigned_id);
		$criteria->compare('status_id',$this->status_id,true);
		$criteria->compare('priority_id',$this->priority_id,true);
		$criteria->compare('created_at',$this->created_at);
		$criteria->compare('edited_at',$this->edited_at);
		$criteria->compare('category_id',$this->category_id);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}

	public function getTracking($length = 8)
	{
		$str = '';
		$charset='ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
		$count = strlen($charset);

		while ($length--)
			$str .= $charset[mt_rand(0, $count - 1)];

		return strtoupper($str);
	}

	public function addMessage(TicketMessage $message)
	{
		/**
		 * @var User $user
		 */
		$user = Yii::app()->getModule('user')->user() ? Yii::app()->getModule('user')->user() : false;

		if ($this->folder_id == TicketFolders::TWITTER)
			$message->setScenario('isTwitter');

		$message->ticket_id = $this->id;
		$message->user_id = $user ? $user->id : 0;

		$message_id = $message->save();

		if ($user)
		{
			$view = new MessageViews();
			$view->message_id = $message_id;
			$view->user_id = $user->id;
			$view->viewed_at = time();

			if ($user->is_worker && $message_id && Settings::get('close_ticket'))
			{
				$ticket = $message->ticket;
				$ticket->is_open = 0;
				$ticket->save(false);
			}
		}

		return $message_id;
	}

	public function sendNotification($to = false, $name = '', $from_email = '', $from_name = '', $account_id = 0)
	{
		if (filter_var($to, FILTER_VALIDATE_EMAIL) === false || $this->stopNotification($to) || !Settings::get('auto_response'))
			return false;

		/**
		 * @var YiiMailMessage $email
		 */
		$email = new YiiMailMessage();
		$email->addTo($to, $name);

		$params = array (
			'{SUBJECT_ID}'     => "ID: [$this->tracking_id]",
			'{TICKET_AUTHOR}'  => $this->author_name,
			'{COMPANY_NAME}'   => Settings::get('company_name'),
			'{TICKET_URL}'     => Yii::app()->createAbsoluteUrl('/ticket/view', array('id' => $this->id, 'access_key' => $this->access_key)),
			'{TICKET_ID}'      => $this->tracking_id,
			'{TICKET_SUBJECT}' => $this->subject,
		);

		$email->template_code = 'new_ticket_notification';
		$email->account_id = $account_id;

		$email->setBody($params);

		$from_email = $from_email ? $from_email : Settings::get('email_address');

		if (!$from_email)
			$from_email = Yii::app()->params['adminEmail'];
		else
			$from_email = array($from_email => $from_name ? $from_name : Settings::get('email_name'));

		$email->setFrom($from_email);

		return sendAppMail($email);
	}

	public function sendAssignNotification()
	{
		/** @var User $assigned_user */
		$assigned_user = User::model()->findByAttributes(array('id' => $this->assigned_id));

		if ($assigned_user->send_notifications)
		{
			/** @var YiiMailMessage $email */
			$email = new YiiMailMessage();
			$email->addTo($assigned_user->email, $assigned_user->profile ? $assigned_user->profile->full_name : '');

			$params = array (
				'{TICKET_SUBJECT}' => $this->subject,
				'{TICKET_ID}'      => $this->id,
				'{TICKET_AUTHOR}'  => $this->author_name,
				'{EMAIL_CONTENT}'  => TicketMessage::showEmail($this->content),
				'{TICKET_URL}'     => Yii::app()->createAbsoluteUrl('/ticket/view', array('id' => $this->id)),
				'{COMPANY_NAME}'   => Settings::get('company_name'),
			);

			$email->template_code = 'ticket_assigned';
			$email->setBody($params);

			$from_email = Settings::get('email_address');

			if (!$from_email)
				$from_email = Yii::app()->params['adminEmail'];
			else
				$from_email = array(Settings::get('email_address') => Settings::get('email_name'));

			// maybe this ticket from external email
			/** @var EmailAccounts $email_account */
			$email_account = EmailAccounts::model()->findByAttributes(array('department' => $this->category_id));
			if (!empty($email_account))
				$from_email = array ($email_account->email_address => $email_account->email_name);

			$email->setFrom($from_email);

			return sendAppMail($email, $email_account);
		}
	}

	public function setViews()
	{
		$user_id = Yii::app()->getModule('user')->user() ? Yii::app()->getModule('user')->user()->id : 0;
		if (!$user_id)
			return true;

		foreach ($this->messages as $message)
		{
			$has_views = MessageViews::model()->countByAttributes(array('user_id' => $user_id, 'message_id' => $message->id));
			if ($has_views)
				continue;

			$view = new MessageViews();
			$view->user_id = $user_id;
			$view->message_id = $message->id;
			$view->viewed_at = time();
			$view->save();
		}
	}

	public function getNotViewedIds()
	{
		$user_id = Yii::app()->getModule('user')->user() ? Yii::app()->getModule('user')->user()->id : 0;

		if (!$user_id)
			return false;

		$ids = Yii::app()->db->createCommand()
			->select('m.id')
			->from(TicketMessage::model()->tableName() . ' m')
			->leftJoin(MessageViews::model()->tableName() . ' mv', 'm.id = mv.message_id and mv.user_id = :user_id', array('user_id' => $user_id))
			->where('mv.id IS NULL AND m.ticket_id = :id', array('id' => $this->id))
			->queryColumn();

		return $ids;
	}

	public function getPersons()
	{
		if (!empty($this->author_name))
			$persons = array (md5($this->author_name) => $this->author_name);
		elseif (!empty($this->user_id))
		{
			$user = $this->getUserInfo($this->user_id);
			$persons[md5($user->profile->full_name)] = $user->profile->full_name;
		}
		else
		{
			$email_prefix = $this->getEmailName($this->author_email);
			$persons = array (md5($email_prefix) => $email_prefix);
		}

		foreach ($this->messages as $message)
		{
			if (!empty($message->user_id))
			{
				$user = $this->getUserInfo($message->user_id);
				$profile_name = $user->profile->full_name;
				$name = empty($profile_name) ? $this->getEmailName($user->email) : $profile_name;

				$persons[md5($name)] = $name;
			}
		}

		if (count($persons) > 1)
		{
			$persons = array_slice($persons, 0, 2);

			foreach ($persons as &$person)
			{
				$names = explode(' ', $person);
				$first_name = array_shift($names);

				if (count($names) > 0) {
					foreach ($names as &$name)
						$name = mb_substr(trim($name), 0, 1, APP_ENCODING) . ".";
				}

				$person = $first_name . ($names ? ' ' . implode(' ', $names) : '');
			}
		}

		$persons_str = $persons ? implode(', ', $persons) : '';

		return $persons_str;
	}

	public function getNormalDate()
	{
		$date = max($this->last_client, $this->last_staff);
		$ticket_year = date('Y', $date);
		$current_year = date('Y');

		if ($date >= strtotime("today"))
			$format = 'H:i';
		else if ($ticket_year != $current_year)
			$format = 'n/j/Y';
		else
			$format = 'M j';

		return date($format, $date);

	}

	public function isViewed()
	{
		static $viewed = array ();

		$user_id = Yii::app()->getModule('user')->user() ? Yii::app()->getModule('user')->user()->id : 0;

		if (!$user_id)
			return false;

		if (isset($viewed[$this->id]))
			return $viewed[$this->id];

		$message_ids = Yii::app()->db->createCommand('SELECT id FROM ' . TicketMessage::model()->tableName() . ' WHERE ticket_id = ' . $this->id)->queryColumn();

		if (!empty($message_ids))
			$view_count = Yii::app()->db->createCommand('SELECT COUNT(*) FROM ' . MessageViews::model()->tableName() . ' WHERE user_id = :user_id AND message_id IN (' . implode(',', $message_ids) . ')')->queryScalar(array(':user_id' => $user_id));
		else
			$view_count = array ();

		$viewed[$this->id] = count($message_ids) === intval($view_count);

		return $viewed[$this->id];
	}

	public function setView($user_id)
	{
		TicketViews::model()->deleteAllByAttributes(array('user_id' => $user_id, 'ticket_id' => $this->id));
		$view = new TicketViews();
		$view->user_id = $user_id;
		$view->ticket_id = $this->id;
		$view->updated_at = time();

		return $view->save();
	}

	public function getAuthorName()
	{
		if ($this->user_id)
		{
			/** @var User $user */
			$user = User::model()->findByPk($this->user_id);

			if (!empty($user) && $user->is_worker)
			{
				$user_name = $user->profile->full_name;
				$user_email = $user->email;
			}
		}

		if (empty($user_email))
		{
			$user_email = $this->author_email;
			$user_name = $this->author_name;
		}

		$email_prefix = $this->getEmailName($user_email);

		if ($this->folder_id == TicketFolders::TWITTER)
			$user_email = "<a href='https://twitter.com/$user_email' target='_blank' title='" . _t('view on twitter') . "'>$user_email</a>";

		if (in_array($this->external_type, [TicketMessage::TYPE_FB_POST, TicketMessage::TYPE_FB_TAG, TicketMessage::TYPE_FB_MESSAGE, TicketMessage::TYPE_FB_FEED]))
			return "<a href='https://www.facebook.com/$user_email' target='_blank' title='" . _t('view on facebook') . "'>$user_name</a>";

		$name = !empty($user_name) ? $user_name : $email_prefix;
		$name .= " (" . ((defined('DEMO') ? _t('hidden for demo') : $user_email)) . ")";

		return $name;
	}

	public function getAssignedName()
	{
		$users = User::model()->findByPk($this->assigned_id);

		if (!empty($users))
			return $users->profile->full_name;

		return _t('none');
	}

	public function getItemCss()
	{
		return 'hd-ticket-' . $this->id;
	}

	public function isStarred()
	{
		$user_id = Yii::app()->getModule('user')->user() ? Yii::app()->getModule('user')->user()->id : 0;

		if (!$user_id)
			return false;

		$favorite_object = TicketFavorite::model()->findByAttributes(array('ticket_id' => $this->id, 'user_id' => $user_id));
		$favorite = !empty($favorite_object) ? '1' : '0';

		return $favorite;
	}

	public function setStar($user_id)
	{
		$favorite = new TicketFavorite();
		$favorite->ticket_id = $this->id;
		$favorite->created_at = time();
		$favorite->user_id = $user_id;

		return $favorite->save();
	}

	/**
	 * @return Ticket[]
	 */
	public function getRelatedTickets()
	{
		$tickets = Ticket::model()->findAll(array(
			'condition' => 'author_email = :author_email AND id != :id',
			'params' => array (':author_email' => $this->author_email, 'id' => $this->id),
			'order' => 'id DESC',
			'limit' => 20,
		));

		return $tickets;
	}

	public function getLastMessage($list_data = true)
	{
		/**
		 * @var TicketMessage $message
		 */
		$message = array_pop($this->messages);
		$message_str = $message->message;

		if ($list_data) {
			$message_str = strip_tags($message_str);
			$message_str = mb_substr($message_str, 0, 150, APP_ENCODING) . ((mb_strlen($message_str, APP_ENCODING) > 150) ? '...' : '');
		}

		return $message_str;
	}

	public function isAssignedToMe()
	{
		$user_id = Yii::app()->getModule('user')->user() ? Yii::app()->getModule('user')->user()->id : 0;

		if (!$user_id)
			return false;

		return $this->assigned_id == $user_id;
	}

	public function setUnread()
	{
		$user_id = Yii::app()->getModule('user')->user() ? Yii::app()->getModule('user')->user()->id : 0;

		if (!empty($user_id))
		{
			$message_ids = Yii::app()->db->createCommand('SELECT id FROM ' . TicketMessage::model()->tableName() . ' WHERE ticket_id = ' . $this->id)->queryColumn();

			if (!empty($message_ids))
				Yii::app()->db->createCommand('DELETE FROM ' . MessageViews::model()->tableName() . ' WHERE message_id IN (' . implode(',', $message_ids) . ') AND user_id = ' . $user_id)->execute();

			return TicketViews::model()->deleteAllByAttributes(array('user_id' => $user_id, 'ticket_id' => $this->id));
		}

		return false;
	}

	/**
	 * @return CDbCriteria $criteria
	 */
	public static function getBaseCriteria($filter = '', $selector = '', $q = '', $folder = '')
	{
		$criteria = new CDbCriteria();
		$criteria->select = 'ticket.*';
		$criteria->alias = 'ticket';

		$user = Yii::app()->getModule('user')->user();
		/** @var AppDbAuthManager $manager */
		$manager = Yii::app()->getAuthManager();

		if ($folder == TicketFolders::INBOX)
			$folder = [TicketFolders::INBOX, TicketFolders::TWITTER, TicketFolders::FACEBOOK];

		$sql = '(1';
		$sql_params = array ();

		if ($filter == 'my' || ($user))
		{
			if ($user->is_worker)
			{
				$sql .= ' AND ((assigned_id = :assign_id)';
				$sql_params[':assign_id'] = $user->id;
			}
			else
			{
				$sql .= ' AND ((ticket.user_id = :user_id)';
				$sql_params[':user_id'] = $user->id;
			}
		}

		if (!empty($user->categories) && $filter != 'my' && $manager->hasCredential($user->id, 'Ticket.Index'))
		{
			$sql .= ' OR (';
			$cat_sql = array();
			foreach ($user->categories as $k => $cat)
			{
				$cat_sql[] = 'category_id = :category_id_' . $k;
				$sql_params[':category_id_' . $k] = $cat->category_id;
			}

			$sql .= (implode(' OR ', $cat_sql) . ')');
		}

		if ($filter == 'my' || ($user))
			$sql .= ' ) ';

		$sql .= ')';

		if ($selector == 'unread' || !empty($q))
			$criteria->join .= ' LEFT JOIN ' . TicketMessage::model()->tableName() . ' as message ON message.ticket_id = ticket.id';

		if ($selector == 'open')
			$sql .= ' AND is_open = 1';
		elseif ($selector == 'closed')
			$sql .= ' AND is_open = 0';
		elseif ($selector == 'unread')
		{
			$criteria->join .= ' LEFT JOIN ' . MessageViews::model()->tableName() . ' as views ON views.message_id = message.id AND views.user_id = :user_id';
			$sql .= ' AND views.message_id IS NULL';
			$criteria->group = 'ticket.id';
			$sql_params[':user_id'] = $user->id;
		}
		elseif ($selector == 'starred')
		{
			$criteria->join = ' JOIN ' . TicketFavorite::model()->tableName() . ' as star ON star.ticket_id = ticket.id AND star.user_id = :user_id';
			$sql_params[':user_id'] = $user->id;
		}

		/**
		 * smart queries
		 */
		if (!empty($q))
		{
			$q = trim($q);

			if (preg_match('/^([A-Z\d]+)$/', $q))
			{
				$sql .= ' AND ticket.tracking_id = :code';
				$sql_params[':code'] = $q;
			}
			elseif (filter_var($q, FILTER_VALIDATE_EMAIL))
			{
				$sql .= ' AND ticket.author_email = :author';
				$sql_params[':author'] = $q;
			}
			elseif (preg_match('/author:([\w@\.]+)/', $q, $matches))
			{
				$sql .= ' AND (ticket.author_email = :author)';
				$sql_params[':author'] = $matches[1];
			}
			else
			{
				$sql .= ' AND (message.message LIKE :q OR ticket.subject LIKE :q)';
				$sql_params[':q'] = "%$q%";
				$criteria->group = 'ticket.id';
			}
		}

		if (is_array($folder))
			$sql .= ' AND folder_id IN (' . implode(', ', $folder) . ')';
		elseif (!empty($folder))
		{
			$sql .= ' AND folder_id = :folder_id';
			$sql_params[':folder_id'] = $folder;
		}

		$criteria->params = $sql_params;
		$criteria->condition = $sql;
		$criteria->with = 'messages';

		return $criteria;
	}

	public function getSubject()
	{
		$subj = empty($this->subject) ? '(no subject)' : $this->subject;

		if (!empty($this->order_id))
			$subj .= ", " . _t('order id', array ('{order_id}' => $this->order_id));

		return $subj;
	}

	public function delete()
	{
		if ($this->folder_id == TicketFolders::TRASH)
			parent::delete();
		else
		{
			$this->folder_id = TicketFolders::TRASH;
			$this->save(false);
		}

		return true;
	}

	/**
	 * @return bool|array
	 */
	public function getOrders()
	{
		$remote_url = Settings::get('api_url');
		$result = true;

		if (!$remote_url || $this->external_type)
			return false;

		$cache = Yii::app()->cache->get(self::CACHE_ORDERS . $this->author_email);

		if ($cache !== false)
			return $cache;

		Yii::import('ext.httpclient.*');

		if (strpos($remote_url, 'http://') === false && strpos($remote_url, 'https://') === false)
			$remote_url = 'http://' . $remote_url;

		$client = new EHttpClient($remote_url);

		$params = array (
			're_action' => 'orders',
			'api_key'   => Settings::get('api_key'),
			'email'     => $this->author_email,
		);

		$client->setParameterGet($params);
		$response = $client->request()->getBody();

		if (!empty($response))
		{
			$result = json_decode($response, true);

			if ($result['result'] == 'no')
				return false;

			if (!empty($result['orders']))
			{
				$orders = $result['orders'];

				foreach ($orders as & $order)
				{
					foreach (array('shipping', 'billing') as $section)
					{
						$data = array_filter($order['addresses'][$section], 'trim');
						$order[$section . '_address'] = implode(', ', $data);
					}
				}

				$result = array($orders, $result['view_all_url']);
			}
		}

		Yii::app()->cache->set(self::CACHE_ORDERS . $this->author_email, $result, 3600);

		return $result;
	}

	public function checkUpload()
	{
		/** @var CUploadedFile[] $attachments */
		$attachments = CUploadedFile::getInstancesByName('attachments');

		if (!empty($attachments))
		{
			foreach ($attachments as $attach)
			{
				if ($attach->size > 1024 * 1024 * TicketMessage::ATTACHMENT_MAX_SIZE)
					$this->addError('attachments', _t('attachment too large'));
			}
		}
	}

	public function getNotify_Email()
	{
		return '';
	}

	public function getNotify_Name()
	{
		return '';
	}

	public function canAccess(User $user)
	{
		$result = false;

		if ($this->assigned_id == $user->id || $this->user_id == $user->id || $user->email == $this->author_email)
			$result = true;

		if (!empty($user->categories) && !empty($this->category_id))
		{
			foreach ($user->categories as $category)
				if ($category->category_id == $this->category_id)
				{
					$result = true;
					break;
				}
		}

		return $result;
	}

	private function getEmailName($email)
	{
		return mb_substr($email, 0, mb_strpos($email, '@', null, APP_ENCODING), APP_ENCODING);
	}

	private function stopNotification($email)
	{
		$stop =
			mb_stripos($email, 'mailer-daemon') !== false
				||
			mb_stripos($email, 'mail-daemon') !== false
				||
			mb_stripos($email, 'postmaster') !== false
				||
			mb_stripos($email, 'listserv') !== false;


		// check that we have sent notification a few minutes ago
		if (!$stop)
		{
			/** @var Ticket $ticket */
			$ticket = Ticket::model()->find(
				array(
					'condition' => 'id != :ticket_id AND author_email = :author_email',
					'order' => 'id DESC',
					'params' => array (
						':ticket_id'    => $this->id,
						':author_email' => $email,
					)
				)
			);

			if ($ticket)
			{
				if (($this->created_at - $ticket->created_at) <= self::MAX_NOTIFY_TIME)
					$stop = true;
			}
		}

		return $stop;
	}

	public function setScopeMessages($ticket_ids = array ())
	{
		if (!empty($ticket_ids))
		{
			$messages_count = Yii::app()->db->createCommand()
			->select('COUNT(*) AS messages, ticket_id')
			->from('tbl_ticket_message')
			->group('ticket_id')
			->where(array ('in', 'ticket_id', $ticket_ids))->queryAll();

			if (!empty($messages_count))
			{
				foreach ($messages_count as $count)
					self::$messagesCount[$count['ticket_id']] = $count['messages'];
			}
		}

		return true;
	}

	public function setScopeFavorites($ticket_ids)
	{
		$user_id = Yii::app()->getModule('user')->user() ? Yii::app()->getModule('user')->user()->id : 0;

		if (!$user_id)
			return false;

		$favorites = Yii::app()->db->createCommand()->
			select('ticket_id')->
			from('tbl_ticket_favorites')->
			where('user_id = :user_id', array (':user_id' => $user_id))->
			andWhere(array ('in', 'ticket_id', $ticket_ids))->
			queryColumn();

		self::$scopeFavorites = empty($favorites) ? array () : array_combine($favorites, $favorites);

		return true;
	}

	public function setScopeViewed($ticket_ids)
	{
		$user_id = Yii::app()->getModule('user')->user() ? Yii::app()->getModule('user')->user()->id : 0;

		if (!$user_id)
			return false;

		$views = Yii::app()->db->createCommand()->
			select('t.id')->
			from('tbl_ticket as t')->
			join('tbl_ticket_message as tm', 'tm.ticket_id = t.id')->
			leftJoin('tbl_message_views as mv', 'mv.message_id = tm.id AND mv.user_id = :user_id', array (':user_id' => $user_id))->
			where(array ('in', 't.id', $ticket_ids))->
			andWhere('mv.id IS NULL')->
			group('t.id')->
			queryColumn();

		self::$scopeViewed = empty($views) ? array () : array_combine($views, $views);

		return true;
	}

	public function getHashedMessageCount()
	{
		return !empty(self::$messagesCount[$this->id]) ? self::$messagesCount[$this->id] : false;
	}

	public function getHashedFavorite()
	{
		return isset(self::$scopeFavorites[$this->id]) ? '1' : '0';
	}

	public function getHashedViewed()
	{
		return empty(self::$scopeViewed[$this->id]);
	}

	public function isFacebook()
	{
		return in_array($this->external_type, [
			TicketMessage::TYPE_FB_POST,
			TicketMessage::TYPE_FB_TAG,
			TicketMessage::TYPE_FB_MESSAGE,
			TicketMessage::TYPE_FB_FEED]
		);
	}

	public function getRecentImages()
    {
        $recent_images = array ();
        foreach ($this->messages as $ticket_message)
        {
            foreach ($ticket_message->attachments as $attach)
            {
                if ($attach->isImage())
                    $recent_images[] = $attach;
            }
        }

        if (!empty($recent_images))
            $recent_images = array_reverse($recent_images, true);

        return array_slice($recent_images, 0, 6);
    }

    public function getApiInfo()
    {
        $api_info =  Settings::get('api_url') ? $this->getOrders() : false;

        if (!empty($api_info) && !empty($api_info[0]) && !empty($this->order_id))
        {
            foreach ($api_info[0] as $k => $order)
                if ($order['id'] == $this->order_id)
                {
                    unset($api_info[0][$k]);
                    $order['ticket_order'] = 1;
                    array_unshift($api_info[0], $order);
                }
        }

        return $api_info;
    }

	/**
	 * @return User
	 */
	public function getUserInfo($user_id)
	{
		if (!self::$usersInfo[$user_id])
			self::$usersInfo[$user_id] = User::model()->findByPk($user_id);

		return self::$usersInfo[$user_id];
	}
}