<?php
/***************************************************************************
 *                                                                          *
 *				   (c) 2013 Re:Desk Development Team    				    *
 *                                                                          *
 * This  is  commercial  software,  only  users  who have purchased a valid *
 * license  and  accept  to the terms of the  License Agreement can install *
 * and use this program.                                                    *
 *                                                                          *
 ****************************************************************************
 * PLEASE READ THE FULL TEXT  OF THE SOFTWARE  LICENSE   AGREEMENT  IN  THE *
 * "copyright.txt" FILE PROVIDED WITH THIS DISTRIBUTION PACKAGE.            *
 ****************************************************************************/

/**
 * This is the model class for table "tbl_ticket_message".
 *
 * The followings are the available columns in table 'tbl_ticket_message':
 * @property integer $id
 * @property string $message
 * @property string $author_email
 * @property string $author_name
 * @property integer $user_id
 * @property integer $assign_id
 * @property integer $ticket_id
 * @property integer $created_at
 * @property integer $is_answered
 * @property integer $is_worker
 * @property integer $moved_to
 * @property integer $moved_from
 * @property integer $external_id
 * @property integer $external_type
 *
 * The followings are the available model relations:
 * @property Ticket $ticket
 * @property MessageAttachment[] $attachments
 */
class TicketMessage extends CActiveRecord
{
	const MESSAGE_SIGN_PREFIX = "<br /><br />";
	const DO_NOT_WRITE_LINE = '-- DO NOT WRITE YOUR REPLY BELOW THIS LINE --';
	// max attachment size in MB
	const ATTACHMENT_MAX_SIZE = 4;

	const TYPE_TWEET = 'tweet';
	const TYPE_DM = 'dm';
	const TYPE_FB_TAG = 'fb_tag';
	const TYPE_FB_POST = 'fb_post';
	const TYPE_FB_FEED = 'fb_feed';
	const TYPE_FB_COMMENT = 'fb_comment';
	const TYPE_FB_MESSAGE = 'fb_message';

	/**
	 * Returns the static model of the specified AR class.
	 * @param string $className active record class name.
	 * @return TicketMessage
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}

	public function beforeSave()
	{
		if ($this->isNewRecord) {

			$user_id = $is_worker = 0;

			if (!defined('CONSOLE_APP'))
			{
				/**
				 * @var User $user;
				 */
				$user = Yii::app()->getModule('user')->user();

				if ($user)
				{
					$user_id = $user->id;
					$is_worker = $user->is_worker;
					if (!$this->author_name)
						$this->author_name = $user->profile->full_name;
					if (!$this->author_email)
						$this->author_email = $user->email;
				}
			}

			$this->user_id = $user_id;
			$this->is_worker = $is_worker;

			if (!$this->author_email)
			{
				/** @var $ticket Ticket */
				$ticket = Ticket::model()->findByPk($_REQUEST['id']);
				$this->author_email = $ticket->author_email;
				$this->author_name = $ticket->author_name;
			}
		}

		return parent::beforeSave();
	}

	public function afterSave()
	{
		$ticket = $this->ticket;

		if ($this->ticket->folder_id == TicketFolders::FACEBOOK)
			$ticket->last_staff = $ticket->last_client = $this->created_at;
		else
		{
			if ($this->is_worker)
				$ticket->last_staff = $this->created_at;
			else
				$ticket->last_client = $this->created_at;
		}

		// Users notifications
		$emails = Yii::app()->db->createCommand('
			SELECT DISTINCT tm.author_email
			FROM ' . $this->tableName() . ' tm
			LEFT JOIN ' . User::model()->tableName() . ' u ON u.id = tm.user_id
			WHERE
				tm.ticket_id = :ticket_id AND tm.author_email != :email AND
				((u.send_notifications = 1 AND u.is_worker = 1) OR u.is_worker = 0 OR u.id IS NULL)')
			->queryAll(true, array(':ticket_id' => $ticket->id, ':email' => $this->author_email));

		// send notification to assigned user
		/*if ($ticket->assigned_id)
		{
			$assigned_user = User::model()->findByPk($ticket->assigned_id);

			if ($assigned_user && $this->author_email != $assigned_user->email && !in_array($assigned_user->email, $emails))
			{
				$assigned_email = $assigned_user->email;

				$add = true;

				if (!empty($emails))
				{
					foreach ($emails as $_address)
						if ($_address['author_email'] == $assigned_email)
							$add = false;
				}

				if ($add)
					$emails[] = array ('author_email' => $assigned_email);
			}
		}*/

		foreach ($emails as $_k => $_v)
		{
			if (filter_var($_v['author_email'], FILTER_VALIDATE_EMAIL) === false)
				unset($emails[$_k]);
		}

		$account = null;

		if (!empty($ticket->account_id))
			/** @var EmailAccounts $account */
			$account = EmailAccounts::model()->findByAttributes(array('id' => $ticket->account_id));

		if (!empty($emails))
		{
			$email = new YiiMailMessage();
			$ticket = $this->ticket;

			if ($this->user_id)
			{
				$user = Yii::app()->getModule('user')->user();

				$sign = $user->profile->sign ? TicketMessage::MESSAGE_SIGN_PREFIX . $user->profile->sign : '';
			}
			else
				$sign = '';

			$from_email = Settings::get('email_address');

			if (!$from_email)
				$from = Yii::app()->params['adminEmail'];
			else
				$from = array(Settings::get('email_address') => Settings::get('email_name'));

			// maybe this ticket from external email
			if (!empty($account))
				$from = array ($account->email_address => $account->email_name);

			$email->setFrom($from);

			$params = array (
				'{TICKET_SUBJECT}' => $ticket->subject,
				'{EMAIL_CONTENT}' => TicketMessage::showEmail($this->message),
				'{TICKET_URL}'     => Yii::app()->createAbsoluteUrl('/ticket/view', array('id' => $ticket->id, 'access_key' => $ticket->access_key))
			);

			$email->template_code = 'new_message';
			$email->setBody($params, 'text/html', APP_ENCODING, $sign, TicketMessage::DO_NOT_WRITE_LINE . "\n\n", ' ' . "ID: [$ticket->tracking_id]");

			$attachments = CUploadedFile::getInstancesByName('attachments');
			if (!empty($attachments))
			{
				foreach ($attachments as $attach)
				{
					$mail_attach = Swift_Attachment::fromPath($attach->tempName);
					$mail_attach->setFilename($attach->name);
					$email->attach($mail_attach);
				}
			}

			foreach ($emails as $email_address)
			{
				$email->addTo($email_address['author_email']);
				sendAppMail($email, $account);
			}
		}

		if ($this->is_worker)
		{
			// its twitter message - should post to twitter
			if ($this->ticket->folder_id == TicketFolders::TWITTER)
			{
				$reply_to = TicketMessage::model()->findByAttributes(
					array(
						'ticket_id' => $this->ticket_id,
						'is_worker' => '0'
					),
					array('order' => 'external_id DESC')
				);

				if (!empty($reply_to))
				{
					$connection = TwitterApi::getConnection();

					if ($reply_to->external_type == TicketMessage::TYPE_TWEET)
					{
						$params = array(
							'in_reply_to_status_id' => $reply_to->external_id,
							'status' => $this->message,
						);

						$url = 'statuses/update';
					}
					else
					{
						$params = array(
							'screen_name' => $reply_to->author_email,
							'text'        => $this->message,
						);

						$url = 'direct_messages/new';
					}

					$result = $connection->post($url, $params);

					if (!empty($result) && $result->id_str)
					{
						$message = TicketMessage::model()->findByPk($this->id);
						$message->saveAttributes(array('external_id' => $result->id_str, 'external_type' => $reply_to->external_type));
					}
				}
			}

			// its facebook message
			if ($ticket->isFacebook() && Settings::get('facebook_page_token'))
			{
				$facebook = FacebookApi::getConnection(Settings::get('facebook_page_token'));

				if (!empty($_REQUEST['reply-to']))
				{
					$message = TicketMessage::model()->findByPk($_REQUEST['reply-to']);

					$comment = $facebook->get('/' . $message->external_id . '/?fields=parent')->getDecodedBody();

					if ($comment['parent'])
						$facebook->post('/' . $comment['parent']['id'] . '/comments', ['message' => $this->message]);

				} elseif ($ticket->external_type == self::TYPE_FB_POST)
					$facebook->post('/' . $ticket->external_id . '/messages', ['message' => $this->message]);
				else
					$facebook->post('/' . $ticket->external_id . '/comments', ['message' => $this->message]);
			}
		}

		$ticket->save(false);
	}

	public function behaviors(){
		return array(
			'HDTimestampBehavior' => array(
				'class' => 'HDTimestampBehavior',
				'createAttribute' => 'created_at',
				'updateAttribute' => 'edited_at',
			)
		);
	}

	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return 'tbl_ticket_message';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.

		$rules = array(
			array('message', 'required'),
			array('user_id, ticket_id, created_at', 'numerical', 'integerOnly' => true),
			array('author_email', 'length', 'max' => 255),
			// The following rule is used by search().
			// Please remove those attributes that should not be searched.
			array('id, message, author_email, user_id, ticket_id, created_at', 'safe', 'on'=>'search'),
			array('attachments', 'checkSize'),
			array('message', 'length', 'max' => 140, 'on' => 'isTwitter'),
		);

		return $rules;
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
			'ticket' => array(self::BELONGS_TO, 'Ticket', 'ticket_id'),
			'attachments' => array (
				self::HAS_MANY,
				'MessageAttachment',
				'message_id',
			),
		);
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => 'ID',
			'message' => 'Message',
			'author_email' => 'Author Email',
			'user_id' => 'User',
			'ticket_id' => 'Ticket',
			'created_at' => 'Created At',
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 * @return CActiveDataProvider the data provider that can return the models based on the search/filter conditions.
	 */
	public function search()
	{
		// Warning: Please modify the following code to remove attributes that
		// should not be searched.

		$criteria=new CDbCriteria;

		$criteria->compare('id',$this->id);
		$criteria->compare('message',$this->message,true);
		$criteria->compare('author_email',$this->author_email,true);
		$criteria->compare('user_id',$this->user_id);
		$criteria->compare('ticket_id',$this->ticket_id);
		$criteria->compare('created_at',$this->created_at);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}

	public function addAttachments()
	{
		/**
		 * @var CUploadedFile[] $attachments
		 */
		$attachments = CUploadedFile::getInstancesByName('attachments');

		if (!empty($attachments))
		{
			/** @var $image CUploadedFile */
			foreach ($attachments as $attach)
			{
				$attach_add = new MessageAttachment();
				$attach_add->file_name = $attach->name;
				$attach_add->file_type = $attach->type;
				$attach_add->message_id = $this->id;
				$attach_add->content = file_get_contents($attach->tempName);
				$attach_add->size = $attach->getSize();
				$attach_add->save();
			}
		}

		return true;
	}

	public function show($short = true)
	{
		if ($short)
		{
			$message = strip_tags($this->message);
			$message = mb_substr($message, 0, 150, APP_ENCODING) . ((mb_strlen($message, APP_ENCODING) > 150) ? '...' : '');
		}
		else
		{
			$message = $this->message;
			$message = htmlspecialchars($message, ENT_NOQUOTES);
			$message = str_replace(array('[quote]', '[/quote]'), array ('<blockquote>', '</blockquote>'), $message);
			$message = preg_replace('#&lt;(/?(?:a|br|p|blockquote))&gt;#', '<\1>', $message);
			$message = str_replace('&amp;', '&', $message);
			// replace domain name into link
			$message = preg_replace_callback("/(?<![http|https]:\/\/)(www\.*(\.[\d\w-]+)+)/i", array ($this, '_replaceDomain'), $message);
			// replace all links
			$message = preg_replace('!(https*://[a-z0-9\./?=\-&@_]+)!i', '<a href="$1" target="_blank">$1</a>', $message);
			// replace quote
			$message = preg_replace("/(<blockquote>\s+)/is", '<blockquote>', $message);
			$message = preg_replace("/(\s+<\/blockquote>)/is", '</blockquote>', $message);
			$message = nl2br($message);
		}

		return $message;
	}

	public static function showEmail($message)
	{
		$replacements = array (
			'<blockquote>' => '<blockquote style="color: #888; border-left: 2px solid #eee ; margin: 10px 20px 10px 10px; padding: 0 0 0 15px;">',
		);

		return str_replace(array_keys($replacements), $replacements, $message);
	}

	public function getNormalDate()
	{
		$date = $this->created_at;
		$ticket_year = date('Y', $this->created_at);
		$current_year = date('Y');
		$append_str = '';
		$days_ago = floor((time() - $date) / (24 * 60 * 60));

		if ($days_ago < 1)
		{
			$format = 'H:i';
			$minutes_ago = ceil((time() - $date) / 60);
			$hours = ceil($minutes_ago / 60);

			if ($minutes_ago < 30)
				$append_str = " (" . $minutes_ago . ($minutes_ago % 10 === 1 ? ' minute' : ' minutes') . ' ago' . ")";
			else
				$append_str = " ($hours " . ($hours % 10 === 1 ? 'hour' : 'hours') . " ago)";
		}
		elseif ($days_ago <= 12)
		{
			$append_str = " ($days_ago " . ($days_ago === 1 ? 'day' : 'days') . " ago)";
			$format = 'M j';
		}
		else if ($ticket_year != $current_year)
			$format = 'n/j/Y';
		else
			$format = 'M j';

		return date($format, $date) . $append_str;
	}

	public function isLastUserMessage()
	{
		if ($this->is_worker)
			return false;

		$messages = array_reverse($this->ticket->messages);

		$was_last = false;
		$is_last = true;

		/**
		 * @var TicketMessage $message
		 */
		foreach ($messages as $message) {
			if ($message->is_worker)
			{
				if (!$was_last)
					continue;
				else
				{
					$is_last = false;
					break;
				}
			}

			$was_last = true;

			if ($message->id == $this->id)
				break;
		}

		return $is_last;
	}

	public function checkSize()
	{
		/** @var CUploadedFile[] $attachments */
		$attachments = CUploadedFile::getInstancesByName('attachments');

		if (!empty($attachments))
		{
			foreach ($attachments as $attach)
			{
				if ($attach->size > 1024 * 1024 * self::ATTACHMENT_MAX_SIZE)
					$this->addError('attachments', _t('attachment is too large'));
			}
		}
	}

	private function _replaceDomain($string) {
		if (preg_match("/^.{1,253}$/", $string[1]) && preg_match("/^[^\.]{1,63}(\.[^\.]{1,63})*$/", $string[1]))
			return 'http://' . $string[1];

		return $string[1];
	}
}