<?php
/***************************************************************************
 *                                                                          *
 *				   (c) 2013 Re:Desk Development Team    				    *
 *                                                                          *
 * This  is  commercial  software,  only  users  who have purchased a valid *
 * license  and  accept  to the terms of the  License Agreement can install *
 * and use this program.                                                    *
 *                                                                          *
 ****************************************************************************
 * PLEASE READ THE FULL TEXT  OF THE SOFTWARE  LICENSE   AGREEMENT  IN  THE *
 * "copyright.txt" FILE PROVIDED WITH THIS DISTRIBUTION PACKAGE.            *
 ****************************************************************************/

/**
 * This is the model class for table "tbl_user_messages".
 *
 * The followings are the available columns in table 'tbl_user_messages':
 * @property string $id
 * @property integer $user_id
 * @property integer $message_id
 * @property integer $assigned_at
 * @property integer $is_viewed
 * @property integer $is_answered
 *
 * The followings are the available model relations:
 * @property Message $message
 * @property Ticket $ticket
 * @property User $user
 */
class UserMessages extends CActiveRecord
{
	/**
	 * Returns the static model of the specified AR class.
	 * @param string $className active record class name.
	 * @return UserMessages the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}

	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return 'tbl_user_messages';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('user_id, message_id, assigned_at', 'required'),
			array('user_id, message_id, assigned_at, is_viewed', 'numerical', 'integerOnly'=>true),
			// The following rule is used by search().
			// Please remove those attributes that should not be searched.
			array('id, user_id, message_id, assigned_at, is_viewed', 'safe', 'on'=>'search'),
		);
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
			'message' => array(self::BELONGS_TO, 'TicketMessage', 'message_id'),
			'user' => array(self::BELONGS_TO, 'User', 'user_id'),
			'ticket' => array(self::BELONGS_TO, 'Ticket', 'ticket_id'),
		);
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => 'ID',
			'user_id' => 'User',
			'message_id' => 'Message',
			'assigned_at' => 'Assigned At',
			'is_viewed' => 'Is Viewed',
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 * @return CActiveDataProvider the data provider that can return the models based on the search/filter conditions.
	 */
	public function search()
	{
		// Warning: Please modify the following code to remove attributes that
		// should not be searched.

		$criteria=new CDbCriteria;

		$criteria->compare('id',$this->id,true);
		$criteria->compare('user_id',$this->user_id);
		$criteria->compare('message_id',$this->message_id);
		$criteria->compare('assigned_at',$this->assigned_at);
		$criteria->compare('is_viewed',$this->is_viewed);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}
}