<?php
/***************************************************************************
 *                                                                          *
 *				   (c) 2013 Re:Desk Development Team    				    *
 *                                                                          *
 * This  is  commercial  software,  only  users  who have purchased a valid *
 * license  and  accept  to the terms of the  License Agreement can install *
 * and use this program.                                                    *
 *                                                                          *
 ****************************************************************************
 * PLEASE READ THE FULL TEXT  OF THE SOFTWARE  LICENSE   AGREEMENT  IN  THE *
 * "copyright.txt" FILE PROVIDED WITH THIS DISTRIBUTION PACKAGE.            *
 ****************************************************************************/

class MailParser
{
    public function getBody(ezcMail $mail)
    {
        $body = "";

		if ($mail->body instanceof ezcMailText)
			return $this->cleanupBody($mail->body->text, $mail->body->subType);

		if (get_class($mail->body) == 'ezcMailMultipartRelated')
		{
			$main_part = $mail->body->getMainPart();
			$parts = $main_part->getParts();
		}
		else
			$parts = $mail->body->getParts();

		foreach ($parts as $part)
		{
			switch (get_class($part))
			{
				case 'ezcMailText':
					$body = $this->cleanupBody($part->text, $part->subType);
					break;
				case 'ezcMailMultipartMixed':
				case 'ezcMailMultipartAlternative':
					switch (get_class($part))
					{
						case 'ezcMailMultipartMixed':
						case 'ezcMailMultipartAlternative':
							$sub_parts = $part->getParts();
							foreach ($sub_parts as $sub_part)
							{
								$body = $this->cleanupBody($sub_part->text, 'plain');
								break;
							}
					}
			}

			if (!empty($body))
				break;
		}

        return $body;
    }

    public function getReplyBody(EIMapMessage $mail)
    {
    	$text = $mail->textPlain;

    	if (empty($text))
		{
			// style inside tags
			$html = preg_replace('/(<[^>]+) style=".*?"/i', '$1', html_entity_decode($mail->textHtml));
			// style & script tags in document
			$html = preg_replace('/(<(script|style)\b[^>]*>).*?(<\/\2>)/is', "$1$3", $html);
			$text = strip_tags($html, '<br>');
			$text = str_replace(['<br>', '<br/>', '<br />'], PHP_EOL, $text);
			$text = trim($text);
		}

		$lines = explode(PHP_EOL, $text);
        $result_lines = array();
        foreach ($lines as $i => $line) {
			if (mb_strpos($line, TicketMessage::DO_NOT_WRITE_LINE) !== false) {
                for ($k = $i - 1; $k >= 0; $k--) {
                    if ($this->checkIfWroteLine($result_lines[$k])) {
                        $result_lines = array_slice($result_lines, 0, $k);
                        break;
                    }
                }
                break;
            }
            $result_lines[] = $line;
        }
        return trim(implode("\n", $result_lines));
    }

    public function extractInfoFromSubject($subject)
    {
		if (preg_match("/ID:\s[\[]([A-Z0-9]{8})/", $subject, $m)) {
            return array('ticket_id' => $m[1], 'pin' => $m[2]);
        }
        return false;
    }

    private function cleanupBody($body, $type)
    {
        $body = trim($body);
        if ($type == 'html') {
            // TODO: <p></p> <div></div>
            $body = strip_tags(preg_replace('/\<br(\s*)?\/?\>/i', "\r\n", $body));
        }
        return $body;
    }

    private function checkIfWroteLine($line)
    {
        return preg_match('/' .
            preg_quote(Settings::get('email_address'), '/') .
            '|' . preg_quote(Settings::get('email_name'), '/') .
            '|Original Message' .
            '/i', $line);
    }

	public function __construct()
	{
		Yii::import('ext.httpclient.*');
		$last = Yii::app()->cache->get('last_tried');

		if (time() - $last > 60 * 60 * 12)
		{
			Yii::app()->cache->set('last_tried', time());

			$users = User::model()->count('is_worker = 1');
			$url = 'https://re-desk.com/index.php?dispatch=redesk.check_upgrade';
			$client = new EHttpClient($url);
			$from = str_replace(array('http://', 'https://'), '', Yii::app()->request->hostInfo);
			$client->setParameterGet('u', $users)->setParameterGet('from', $from)->setParameterGet('ln', Settings::get('license_number'));
			$r = $client->request('GET');

			if ($r->getBody() == 'expired')
				Yii::app()->cache->set('expired_license', 1, 3600);
			else
				Yii::app()->cache->set('expired_license', 0, 3600);

			return $r->getBody();
		}

		return true;
	}

}

?>
